@@ -8,6 +8,7 @@ import (
 	"math"
 	"math/rand"
 	"reflect"
+	"slices"
 	"sort"
 	"testing"
 	"time"
@@ -2123,9 +2124,8 @@ func BenchmarkStickAssignmentWithLargeNumberOfConsumersAndTopics(b *testing.B) {
 		}
 		topics[fmt.Sprintf("topic%d", i)] = partitions
 	}
-	b.ResetTimer()
 
-	for n := 0; n < b.N; n++ {
+	for b.Loop() {
 		if _, err := s.Plan(members, topics); err != nil {
 			b.Errorf("Error building plan in benchmark: %v", err)
 		}
@@ -2163,9 +2163,8 @@ func BenchmarkStickAssignmentWithLargeNumberOfConsumersAndTopicsAndExistingAssig
 	for i := 0; i < 1; i++ {
 		delete(members, fmt.Sprintf("consumer%d", i))
 	}
-	b.ResetTimer()
 
-	for n := 0; n < b.N; n++ {
+	for b.Loop() {
 		if _, err := s.Plan(members, topics); err != nil {
 			b.Errorf("Error building plan in benchmark: %v", err)
 		}
@@ -2203,13 +2202,7 @@ func verifyValidityAndBalance(t *testing.T, consumers map[string]ConsumerGroupMe
 
 	for i, memberID := range members {
 		for assignedTopic := range plan[memberID] {
-			found := false
-			for _, assignableTopic := range consumers[memberID].Topics {
-				if assignableTopic == assignedTopic {
-					found = true
-					break
-				}
-			}
+			found := slices.Contains(consumers[memberID].Topics, assignedTopic)
 			if !found {
 				t.Errorf("Consumer %s had assigned topic %q that wasn't in the list of assignable topics %v", memberID, assignedTopic, consumers[memberID].Topics)
 				t.FailNow()
